from pydantic import BaseModel, Field, ConfigDict, field_validator

from .utils import convert_to_bytes


class Config(BaseModel):
    host: str
    port: int
    buffer_size: int = Field(alias='buffer-size', default=4 * 1024)  # 4KB
    max_size: int = Field(alias='max-size', default=2 * 1024 ** 3)  # 2GB
    max_data: int = Field(alias='max-data', default=0)  # unlimited
    api_keys: set[str] = Field(alias='api-keys')
    database: str
    log: str = '-'

    model_config = ConfigDict(extra='forbid')

    @field_validator('buffer_size', 'max_size', 'max_data', mode='before')
    @classmethod
    def convert_size(cls, value: int | str) -> int:
        return convert_to_bytes(value)

    @field_validator('api_keys', mode='before')
    @classmethod
    def check_if_key_in_file(cls, value: str | list[str]) -> set[str]:
        if isinstance(value, str):
            with open(value) as file:
                return set(filter(lambda x: x.strip() != '', file.read().splitlines()))
        return set(value)


class TestDataBody(BaseModel):
    api_key: str
    size: int

    model_config = ConfigDict(extra='forbid')

    @field_validator('size', mode='before')
    @classmethod
    def convert_size(cls, value: str) -> int:
        return convert_to_bytes(value)
