import sys
import argparse
import os
from os.path import exists

import uvicorn

from .utils import save_database
from .custom_types import Config


# Setup Parser
def parse_cli_arguments(argv: list[str]) -> argparse.Namespace:
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '-c',
        '--config',
        type = argparse.FileType('r'),
        default = './config.json',
        help = 'Path to config file in JSON format.'
    )

    return parser.parse_args(argv)


def run(host: str, port: int, api_keys: set[str], max_size: int, max_data: int, database: str, buffer_size: int, reload: bool = False):
    if not exists(database) or os.stat(database).st_size == 0:
        save_database(database, {'data-used': 0})

    # app = create_api(api_keys, max_size, max_data, database, buffer_size)

    uvicorn.run("testdata.app:app", host=host, port=port, reload=reload)


def main():
    args = parse_cli_arguments(sys.argv[1:])
    config = Config.model_validate_json(args.config.read())
    run(**config.model_dump(exclude={'log'}), reload=True)
